
'-----------------------------------------------------------------------------------------------
'    Name:   SMEDIA_DOC.INC    /   Include File
'    Date:   July-1-2002
'    Type:   Tiger-BASIC(tm) Source-Code
' Purpose:   Documentation for SMartMedia(tm) FLASH-Card adaption to BASIC Tiger computers
'
' (C) - Copyright Wilke Technology GmbH, 52070 Aachen, Germany, www.wilke-technology.com
'-----------------------------------------------------------------------------------------------
'
'
'
'
'		!-------------########------------!
'		!             ########            !
'       P6-0   (1)                               (44)  Vcc (5V)
'       P6-1   (2)                               (43)  Batt.Backup Input: 2,7 ... 3,6 V
'       P6-2   (3)                               (42)  A/D-Ref-in
'       P6-3   (4)                               (41)  AGND
'       P6-4   (5)          TINY - Tiger         (40)  Analog-IN-3
'       P6-5   (6)                               (39)  Analog-IN-2
'       P6-6   (7)             Pinout            (38)  Analog-IN-1
'       P6-7   (8)                               (37)  Analog-IN-0
'     P7-0     (9)                               (36)  P41: PC-Mode=0, Run-Mde=1
'     P7-1    (10)                               (35)  - reserved -
' PWM P7-2    (11)                               (34)  Alarm-out
' PWM P7-3    (12)                               (33)     P37
'       P8-0  (13)                               (32)     P36
'       P8-1  (14)                               (31)     P35
'       P8-2  (15)                               (30)     P34
'       P8-3  (16)                               (29)     P33
' Pi-1  P8-4  (17)                               (28)  P95, RTS0-out
'       P8-5  (18)                               (27)  P94, RxD1-IN
' Po-1  P8-6  (19)                               (26)  P93, TxD1-out
'       P8-7  (20)                               (25)  P92, CTS0-IN
'   Reset-IN  (21)                               (24)  P91, RxD0-IN
'        GND  (22)                               (23)  P90, TxD0-out
'		!                                 !
'		!---------------------------------!
'
'
'----------------------------------------------------------------------------------------
'
'	Smart-media Connection, default pin use:
'
'	<- BASIC-Tiger ->	<------- Function -------->
'		P6		BUS  =	8 Data I/O lines
'
'		P8-0		-RE  =	read enable
'		P8-1		-WE  =	write enable
'		P8-2		ALE  =	address latch enable
'		P8-3		CLE  =	command latch enable
'		P8-4		-
'		P8-5		-
'		P8-6		-CE  =	chip enable
'		P8-7		R/-B Input (optional)  -  not used by device driver
'
'
'
'  Setting different pins to control a SmartMedia FLASH Card.
'
' Use INSTALL_DEVICE to assign different Tiger I/O-Pins to control a Smart Media Device:
'
'                                        Ctrl
'                                      Bus  RE WE CE CLE ALE, speed
' INSTALL_DEVICE #9, "SMEDIA_128MB.TDD",6,8,0, 1, 6,  3,  2     10 ' install SmartMedia with expl. settings
'                                       ! ! !  !  !   !   !      !
'                                       ! ! !  !  !   !   !      !<-- Speed-Reduction: 0,1 full speed
'                                       ! ! !  !  !   !   !                            2...31: reduced speed
'                                       ! ! !  !  !   !   !
'                                       ! ! !  !  !   !   !<-- Bit-No of ALE Signal in CTRL-Port
'                                       ! ! !  !  !   !
'                                       ! ! !  !  !   !<-- Bit-No of CLE Signal in CTRL-Port
'                                       ! ! !  !  !
'                                       ! ! !  !  !<-- Bit-No of Chip Enable Signal in CTRL-Port
'                                       ! ! !  !
'                                       ! ! !  !<-- Bit-No of WRITE Enable Signal in CTRL-Port
'                                       ! ! !
'                                       ! ! !<-- Bit-No of WRITE Enable Signal in CTRL-Port
'                                       ! !
'                                       ! !<-- Port for CTRL-Pins
'                                       !
'                                       !<-- BUS-Port (8-bit Bus) for DATA- and Command-I/O
'
'
'
'
' Device Drivers:
'
'	Driver Name:	    Comment:
'
'	SMEDIA_16MB.TDD	  = avaliable buffer space enough for up to  16 MByte Smart Media
'	SMEDIA_32MB.TDD	  = avaliable buffer space enough for up to  32 MByte Smart Media
'	SMEDIA_64MB.TDD	  = avaliable buffer space enough for up to  64 MByte Smart Media
'	SMEDIA_128MB.TDD  = avaliable buffer space enough for up to 128 MByte Smart Media
'
'----------------------------------------------------------------------------------------
'
'	SmartMedia Adapter Pinout with Signalnames:     3.3V / 5V Adapter Types
'	-------------------------------------------
'	       (Default pins of device driver)
'
'		     26-pin connector
'
'                GND   ( 1)  ( 2)  Vcc = 5V Input
'          P83   CLE   ( 3)  ( 4)  -CE      P86
'          P82   ALE   ( 5)  ( 6)  -RE      P80
'          P81   -WE   ( 7)  ( 8)  R/-B     P87 (input)
'                -WP   ( 9)  (10)  GND
'          P60  I/O-0  (11)  (12)  <--  Future: LVD (low-voltage-detect)
'          P61  I/O-1  (13)  (14)  I/O-7    P67
'          P62  I/O-2  (15)  (16)  I/O-6    P66
'          P63  I/O-3  (17)  (18)  I/O-5    P65
'                GND   (19)  (20)  I/O-4    P64
'                GND   (21)  (22)  Vcc = 5V Input
'                  -   (23)  (24)  -
'                  -   (25)  (26)  -
'
'
'	-WP (write-protect) connected to RESET and wired and-ed write-protect-label
'
'	Pins 23 ... 26 check SmartMedia Adapter Datasheet.
'
'	When using the SmartMedia Adapter +5V / GND can be used for supplying the Card with
'	energy. Though, current SmartMediaCards are 3.3V devices.
'
'----------------------------------------------------------------------------------------
'
' USER-Function Code Map for Smart Media Device-Driver - check latest release information
' and examples for implemented functions
'
'
' Func:  <------------ GET ----------->  <------------ PUT ----------->
'   0                  ...               .             ...
'   1                  ...               Set IO-CTRL-Block to GLOBAL-String
'   2    Mount-SMedia: ID+Infoset+iBlks  .             ...
'   3    UN-Mount-SMedia - S-Media raus  .             ...
'   4    Reset Smart-Media Card          .             ...
'   5    Read-SMedia-Status-Byte         .             ...
'   6    GET ID + Infoset about SMedia   .             ...
'   7    GET invalid-Block-Table         .             ...
'   8                  ...               .             ...
'   9                  ...               .             ...
'
'  10  - OPEN-for-Read-Seq, ADR          OPEN-for-Write-Seq, ADR       +
'  11  - READ-Next-DATA-Seq              Write-NEXT-Data-Seq           +
'  12  - CLOSE-for-Reading (opt)         CLOSE-for-Write               +
'  13                  ...               .             ...
'  14                  ...               .             ...
'  15                  ...               .             ...
'  16                  ...               .             ...
'  17    ERASE Block abs (not waiting)   ERASE Block abs (not waiting) +
'  18  + READ-1-DATA-Page-abs.           Write-1-Data-Page-abs         +
'  19  + READ-1-DATA-Page-abs.           Write-1-Data-Page-abs + SPARE
'
'  20  + READ-1-SPARE-Page-abs.          Write-1-SPARE-Page-abs
'  21  + READ-multi-SPARE-Page-abs.      .             ...
'  22                  ...               .             ...
'  23                  ...               .             ...
'  24    READ & Check Spare-Field empty  .             ...
'  25    READ & Check PAGE empty         .             ...
'  26    READ-Find-first-empty-Block     .             ...
'  27    READ-Find-first-empty-Page-in-Blk             ...
'
'----------------------------------------------------------------------------------------

