'----------------------------------------------------------------------------------------
'    Name:  Sd_Card_Spi_D.Inc
'    Type:  Tiger-BASIC(tm) Source Code (SD Card Access)
' Release:  1.3
' Purpose:  SD Card Access - Definitions relevant for SD Card direct access routines
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (English)
'	email: support@wilke.de              (German)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (English)
'	www.wilke.de                         (German)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------


#ifndef __SD_CARD_SPI_D
#define __SD_CARD_SPI_D

''#ifdef TIGER_1
#define NCR 8 '' For Tiger_1 NCR=8	
''#else
''#define NCR 7 '' For Tiger_2 NCR=7
''#endif

'' SD Card SPI Commands are defined in the SanDisk SD Card Product Manual v1.9
'' section 5.2.2.1 (page 91 of PDF)
'' http:''www.sandisk.com/pdf/oem/ProdManualSDCardv1.9.pdf

'' SD Command Codes
#define SD_CM_GO_IDLE_STATE	    	0	'' Resets the SD Card
#define SD_CM_SEND_OP_COND	     	1	'' Activates the card's initialization process
#define SD_CM_SEND_CSD              	9   	'' Asks card to send card-specific data
#define SD_CM_SEND_CID		    	10	'' Asks card to send card identification
#define SD_CM_STOP_TRANSMISSION		12	'' Forces card to stop transmission during multi-block read
#define SD_CM_SEND_STATUS	    	13	'' Asks card to send its status register.
#define SD_CM_SET_BLOCKLEN	     	16	'' Selects block length for all subsequent block commands (default is 512)
#define SD_CM_READ_SINGLE_BLOCK		17	'' Reads a block of the size specified by SET_BLOCKLEN
#define SD_CM_READ_MULTIPLE_BLOCK	18	'' Continuously transfers data until interrupted by STOP_TRANSMISSION
#define SD_CM_WRITE_BLOCK	    	24	'' Writes a block of the size specified by SET_BLOCKLEN
#define SD_CM_WRITE_MULTI_BLOCK 	25	'' Continuously writes blocks of data until a stop transmission token is sent instead of start block token.
#define ACMD41_SD_SEND_OP_COND      	41  	'' Activates the card's initialization process
#define SD_APP_CMD                  	55  	'' Notifies the card that the next command is an application spacific command rather than a standard command.
#define SD_CM_READ_OCR              	58  	'' Reads the OCR register of a card.


'' R1 Response Codes (from SD Card Product Manual v1.9 section 5.2.3.1)
#define SD_R1_IN_IDLE_STATE    		10000001b	'' The card is in idle state and running initializing process.
#define SD_R1_ERASE_RESET      		00000010b	'' An erase sequence was cleared before executing because of an out of erase sequence command was received.
#define SD_R1_ILLEGAL_COMMAND  		00000100b	'' An illegal command code was detected
#define SD_R1_COM_CRC_ERROR    		00001000b	'' The CRC check of the last command failed.
#define SD_R1_ERASE_SEQ_ERROR  		00010000b	'' An error in the sequence of erase commands occured.
#define SD_R1_ADDRESS_ERROR    		00100000b	'' A misaligned address, which did not match the block length was used in the command.
#define SD_R1_PARAMETER		     	01000000b	'' The command's argument (e.g. address, block length) was out of the allowed range for this card.
'' R1 bit (1<<7) is always 0
#define SD_START_BLOCK              	11111110b   	'' For Single Block Read, Single Block Write and Multiple Block Read: First byte: Start Block


'' ERROR CODES
#define ERROR_GO_TO_IDLE_STATE		1
#define ERROR_SEND_OP_COND		2
#define ERROR_ACMD41			3
#define ERROR_SET_BLOCKLEN		4
#define ERROR_INITIALIZATION		5
#define ERROR_READ_SINGLE_BLOCK		6
#define ERROR_READ_MULTIPLE_BLOCK	7
#define ERROR_STOP_TRANSMISSION		8
#define ERROR_WRITE_SINGLE_BLOCK	9
#define ERROR_WRITE_MULTIPLE_BLOCK	10
#define ERROR_GET_CSD			11
#define ERROR_GET_CID			12
#define ERROR_GET_OCR			13
#define ERROR_ACMD13			14
#define ERROR_FIND_NCR			15
#define ERROR_CARD_DETECT		16
#define ERROR_CARD_WP			17


'' Complete SD Command Blocks
#define SD_CM_GO_IDLE_STATE_COMMAND	"40 00 00 00 00 95"%
#define SD_CM_SEND_OP_COND_COMMAND	"41 00 00 00 00 ff"%
#define SD_APP_CMD_COMMAND		"77 00 00 00 00 ff"%
#define SD_SEND_STATUS_COMMAND		"4D 00 00 00 00 ff"%
#define SD_SET_BLOCK_LEN_COMMAND	"50 00 00 00 00 ff"%
#define SD_SEND_CSD_COMMAND		"49 00 00 00 00 ff"%
#define SD_READ_OCR_COMMAND		"7A 00 00 00 00 ff"%
#define SD_SEND_CID_COMMAND		"4A 00 00 00 00 ff"%
#define SD_CMD1_SEND_OP_COND_COMMAND	"01 00 00 00 00 ff"%
#define SD_CMD55_APP_CMD_COMMAND	"55 00 00 00 00 ff"%


'' Milliseconds to wait for pre-defined response while sending a command to SD Card
#define SD_READ_RESPONSE_TIMEOUT 	100

'' Size of SD Card Command Block
#define CMD_BLOCK_SIZE		        6
'' Size of the buffer to read a block from SD Card
#define READ_BL_LEN_MAX             	512

'' Size of the CSD Register of SD Card
#define SD_CSD_REGISTER_SIZE		16
'' Size of the OCR Register of SD Card
#define SD_OCR_REGISTER_SIZE		4
'' Size of the CID Register of SD Card
#define SD_CID_REGISTER_SIZE		16

'' Size of the Status Block of SD Card
#define SD_STATUS_BLOCK_SIZE		64

'' Dummy clock cycles
#define CLOCK_CYCLES 			0FFFFFFFFh


#endif  '' __SD_CARD_SPI_D

