'----------------------------------------------------------------------------------------
'    Name:  Fs_Dat_I.Inc
'    Type:  Tiger-BASIC(tm) Source Code (Basic Tiger File System)
' Release:  1.3
' Purpose:  BTFS - Implementation of date and time conversions
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (English)
'	email: support@wilke.de              (German)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (English)
'	www.wilke.de                         (German)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------

#ifndef __FS_DAT_I_INC
#define __FS_DAT_I_INC

#include ufunc3.inc

array MDays(12) of long                 ' number of days in every month

#ifdef CORRECT_RTC
'' variables for time correcting
byte bTimeRecentlyUpdated, bForceRtcReadActive
long lCurrentTime, lPrevForcedReadTime
#endif	'' CORRECT_RTC

'---------------------------------------------------------------------
' Subroutine: 	Converts date and time into time stamp in seconds
'		(counted from 1.1.1980)
'    Input: 	Sec  <byte> = second
'             	Min  <byte> = minute
'             	Hour <byte> = hour
'             	Day  <byte> = day
'             	Mon  <byte> = month
'             	Year <long> = year
'    Output: 	X    <long> = seconds counted from 1.1.1980
'                           up to input date and time
'---------------------------------------------------------------------
sub lDateDosToTig( var long X; long Sec, Min, Hour, Day, Mon, Year )
  long Timezone                         ' s-difference local time and GT
  long I, Days, Hours                   '
  Timezone = 0                          '
  X = Timezone                          '
  I = Year - START_YEAR                 '
  X = X + ((I shr 2) * SEC_PER_4_YEARS) '
  X = X + ((I bitand 3) * SEC_PER_365_DAYS)
  if (I bitand 3) > 0 then              '
    X = X + SEC_PER_DAY                 '
  endif                                 '
  Days = 0                              '
  I = Mon - 1                           '
  while (I > 0)                         '
    I = I - 1                           '
    Days = Days + MDays(I)              '
  endwhile                              '
  Days = Days + Day - 1                 '
  if ((Mon > 2) and (Year bitand 3) = 0) then
    Days = Days + 1                     '
  endif                                 '
  Hours = Days * 24 + Hour              '
  if ((Year bitand 3) = 0)  then        ' summer time correction
    if (Hours > 2184 and Hours < 6600) then ' leap year
      Hours = Hours - 1                 '
    endif                               '
  else                                  '
    if (Hours > 2160 and Hours < 6576) then ' summer time correction
      Hours = Hours - 1                 '
    endif                               '
  endif                                 '
  X = X + Hours * SEC_PER_HOUR          '
  X = X + 60 * Min + Sec                '
end                                     '


'---------------------------------------------------------------------
' Subroutine: Converts time stamp in seconds (counted from 1.1.1980) into
'             corresponding date and time in usual form
'    Input:   Seconds <long> = seconds counted from 1.1.1980
'    Output:  Sec     <byte> = seconds
'             Min     <byte> = minutes
'             Hour    <byte> = hours
'             Day     <byte> = day
'             Mon     <byte> = month
'             Year    <long> = year
'---------------------------------------------------------------------
sub lDateTigToDos( long Seconds; var long Sec, Min, Hour, Day, Mon, Year )
  long Time                             '
  long Timezone                         ' s-difference local time and GT
  Timezone = 0                          '
  Time = Seconds - Timezone             '
  Sec = mod (Time,60)                   '
  Time = Time / 60                      '
  Min = mod (Time,60)                   '
  Time = Time / 60                      '
  Year = START_YEAR + ((Time / (HOURS_PER_4_YEARS)) SHL 2)
  Time = mod (Time,HOURS_PER_4_YEARS)   '
  if (Time >= HOURS_PER_LEAPYEAR) then  '
    Time = Time - HOURS_PER_LEAPYEAR    '
    Year = Year + 1                     '
    Year = Year + (Time / HOURS_PER_YEAR)
    Time = mod (Time,HOURS_PER_YEAR)    '
    if (Time > 2160 and Time < 6576) then
      Time = Time + 1                   '
    endif                               '
  else                                  '
    if (Time > 2184 and Time < 6600) then
      Time = Time + 1                   '
    endif                               '
  endif                                 '
  Hour = mod (Time,24)                  '
  Time = Time / 24                      '
  Time = Time + 1                       '
  if ((Year bitand 3) = 0 and (Time = 60)) then
    Mon = 2                             '
    Day = 29                            '
  else                                  '
    if ((Year bitand 3) = 0 and (Time > 60)) then
      Time = Time - 1                   '
    endif                               '
    Mon = 0                             '
    while (MDays(Mon) < Time)           '
      Time = Time - MDays(Mon)          '
      Mon = Mon + 1                     '
    endwhile                            '
    Mon = Mon + 1                       '
    Day = Time                          '
  endif                                 '
end                                     '


' Bits	Range	Translated Range  Valid Range	Description
' 0..4	0..31	0..31 		  1..28 	up to 1..31 Day
' 5..8	0..15	0..15 		  1..12 	Month
' 9..15	0..127	1980..2107 	  1980..2107 	Year, add 1980 to convert
sub wPackDate( long lpDay, lpMonth, lpYear; var word wpvPackedDate )
  wpvPackedDate = ( lpDay bitand 01fh ) bitor &
    ( ( lpMonth bitand 0fh ) shl 5 ) bitor &
    ( ( ( lpYear - START_YEAR ) bitand 07fh ) shl 9 )
end


' Bits	Range	Translated Range  Valid Range	Description
' 0..4	0..31	0..31 		  1..28 	up to 1..31 Day
' 5..8	0..15	0..15 		  1..12 	Month
' 9..15	0..127	1980..2107 	  1980..2107 	Year, add 1980 to convert
sub lUnPackDate( word wpPackedDate; var long lpvDay, lpvMonth, lpvYear )
  lpvDay = wpPackedDate bitand 01fh
  lpvMonth = ( wpPackedDate shr 5 ) bitand 0fh
  lpvYear = ( ( wpPackedDate shr 9 ) bitand 07fh ) + START_YEAR
end


' Bits 	  Range	  Translated Range  Valid Range   Description
' 0..4 	  0..31   0..62 	    0..59 	  Seconds/2
' 5..10   0..63   0..63 	    0..59 	  Minutes
' 11..15  0..31   0..31 	    0..23 	  Hours
sub wPackTime( long lpSec, lpMin, lpHour; var word wpvPackedTime )
  wpvPackedTime = ( ( lpSec shr 1 ) bitand 01fh ) bitor &
    ( ( lpMin bitand 03fh ) shl 5 ) bitor &
    ( ( lpHour bitand 01fh ) shl 11 )
end


' Bits 	  Range	  Translated Range  Valid Range   Description
' 0..4 	  0..31   0..62 	    0..59 	  Seconds/2
' 5..10   0..63   0..63 	    0..59 	  Minutes
' 11..15  0..31   0..31 	    0..23 	  Hours
sub lUnPackTime( word wpPackedTime; var long lpvSec, lpvMin, lpvHour )
  lpvSec = ( wpPackedTime bitand 01fh ) shl 1
  lpvMin = ( wpPackedTime shr 5 ) bitand 03fh
  lpvHour = ( wpPackedTime shr 11 ) bitand 01fh
end


' initialize global vars used during time/date calculation
sub vDateVarIni()                       '
  MDays(0) = 31                         '
  MDays(1) = 28                         '
  Mdays(2) = 31                         '
  MDays(3) = 30                         '
  MDays(4) = 31                         '
  MDays(5) = 30                         '
  MDays(6) = 31                         '
  MDays(7) = 31                         '
  MDays(8) = 30                         '
  MDays(9) = 31                         '
  MDays(10) = 30                        '
  MDays(11) = 31                        '
end                                     '


' get current date and time in FAT form
sub wGetCurrentDateTime( var word wpvCurDate, wpvCurTime )
  long llSec, llMin, llHour, llDay, llMonth, llYear, llSeconds

  '' get the current value of tiger real time clock (works only for X and C modules)
''''  get #RTC, 1, llSeconds
  call lGetCurrentRtcTime( llSeconds )
  '' convert the time-date stamp (number of seconds from 01.01.1980) into
  '' separated values
  call lDateTigToDos( llSeconds, llSec, llMin, llHour, llDay, llMonth, llYear )
  '' pack date to use in dir entry
  call wPackDate( llDay, llMonth, llYear, wpvCurDate )
  '' pack time to use in dir entry
  call wPackTime( llSec, llMin, llHour, wpvCurTime )
end


sub lGetCurrentRtcTime( var long lpvCurrentSeconds )
#ifdef CORRECT_RTC
  lpvCurrentSeconds = lCurrentTime
#else
  get #RTC, 1, lpvCurrentSeconds
#endif
end


#ifdef CORRECT_RTC
sub vInitRtc()
  lCurrentTime = 0
  lPrevForcedReadTime = 0
  bForceRtcReadActive = TRUE
  run_task ReadRtc
''  set_task_prio ReadRtc, 64
end


'' force rtc driver to read physically the time from rtc chip
'' do validity check repeating the reading process as far as
''   the time value were in the validity interval
''   TIME_VALIDITY_LOOPS times one after another
task ForceRtcRead
  byte blTimeCorrect
  long llRtcState, llCurTime, llPrevTime, llLoopCnt

  bForceRtcReadActive = TRUE
  blTimeCorrect = FALSE
''''  print #LCD,"<1BH>A<0><2><F0H>"; "Forcing...";
''''  print #LCD,"<1BH>A<11><2><F0H>"; "       ";

  while blTimeCorrect = FALSE
    blTimeCorrect = TRUE
    llLoopCnt = 0
    while llLoopCnt < TIME_VALIDITY_LOOPS
      put #RTC, #2, "dummy"			' force RTC reading

      get #RTC, #0, #UFCI_RTC_STAT1, 1, llRtcState  	' read RTC state
      while llRtcState = RTC_BUSY            		' while RTC is busy
        get #RTC, #0, #UFCI_RTC_STAT1, 1, llRtcState 	' read RTC state
      endwhile

      get #RTC, 0, llCurTime		' read time value
      if llLoopCnt > 0 then
        if llCurTime > llPrevTime + TIME_VALIDITY_INTERVAL or llCurTime < llPrevTime then
          blTimeCorrect = FALSE
        else
        endif
      endif
      llPrevTime = llCurTime

      llLoopCnt = llLoopCnt + 1
      wait_duration 1000
    endwhile
  endwhile
  if lCurrentTime = 0 or lCurrentTime > llCurTime + 2 or &
    lCurrentTime < llCurTime - 2 then
''''      print #LCD,"<1BH>A<11><2><F0H>"; abs( lCurrentTime - llCurTime );
      lCurrentTime = llCurTime
      bTimeRecentlyUpdated = TRUE
  endif
  lPrevForcedReadTime = lCurrentTime
  bForceRtcReadActive = FALSE
''''  print #LCD,"<1BH>A<0><2><F0H>"; "Forced    ";
  exit_task ForceRtcRead
end


'' rtc chip is physically read every 1000 sec,
'' otherwise the value of the time counter is simulated by means of ticks() function;
task ReadRtc
  long llTicks, llDiffTicks
  bTimeRecentlyUpdated = FALSE
  bForceRtcReadActive = FALSE
  llTicks = ticks()
  while FALSE = FALSE
    if bTimeRecentlyUpdated = TRUE then
      llDiffTicks = diff_ticks(llTicks)
''      llTicks = ticks() + llDiffTicks
      llTicks = ticks()
      bTimeRecentlyUpdated = FALSE
    else
      if bForceRtcReadActive = FALSE and ( lCurrentTime = 0 or &
        lCurrentTime - lPrevForcedReadTime >= FORCE_READ_PAUSE ) then
          '' force rtc driver to read physically the time from rtc chip
          run_task ForceRtcRead
      else
        llDiffTicks = diff_ticks(llTicks)
        if llDiffTicks >= MS_PER_SEC then
          llDiffTicks = mod( llDiffTicks, MS_PER_SEC )
          '' simulate time counting
          lCurrentTime = lCurrentTime + 1
          llTicks = ticks() - llDiffTicks
        endif
      endif
    endif
  endwhile
end
#endif	'' CORRECT_RTC

#endif  '' __FS_DAT_I_INC

