'----------------------------------------------------------------------------------------
'    Name:  Fs_Ecc_I.Inc
'    Type:  Tiger-BASIC(tm) Source Code (Basic Tiger File System)
' Release:  1.3
' Purpose:  BTFS - Implementation of ECC calculation for SmartMedia
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (English)
'	email: support@wilke.de              (German)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (English)
'	www.wilke.de                         (German)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------


#ifndef __FS_ECC_I_INC
#define __FS_ECC_I_INC

#define SM_ECC_TABLE_SIZE	256
#define SM_ECC_PAGE_SIZE	256

#define BIT7 		80h
#define BIT6 		40h
#define BIT5 		20h
#define BIT4 		10h
#define BIT3 		08h
#define BIT2 		04h
#define BIT1 		02h
#define BIT0 		01h
#define BIT1BIT0 	03h
#define BIT23 		00800000h
#define MASK_CPS 	3fh
#define CORRECTABLE 	00555554h


array bSMECCTable(SM_ECC_TABLE_SIZE) of byte
string sSMECCTableInit$(SM_ECC_TABLE_SIZE)

sub vSMInitECCTable()
  long llIndx
  '' CP0-CP5 code table
  sSMECCTableInit$ = "&
00 55 56 03 59 0C 0F 5A 5A 0F 0C 59 03 56 55 00&
65 30 33 66 3C 69 6A 3F 3F 6A 69 3C 66 33 30 65&
66 33 30 65 3F 6A 69 3C 3C 69 6A 3F 65 30 33 66&
03 56 55 00 5A 0F 0C 59 59 0C 0F 5A 00 55 56 03&
69 3C 3F 6A 30 65 66 33 33 66 65 30 6A 3F 3C 69&
0C 59 5A 0F 55 00 03 56 56 03 00 55 0F 5A 59 0C&
0F 5A 59 0C 56 03 00 55 55 00 03 56 0C 59 5A 0F&
6A 3F 3C 69 33 66 65 30 30 65 66 33 69 3C 3F 6A&
6A 3F 3C 69 33 66 65 30 30 65 66 33 69 3C 3F 6A&
0F 5A 59 0C 56 03 00 55 55 00 03 56 0C 59 5A 0F&
0C 59 5A 0F 55 00 03 56 56 03 00 55 0F 5A 59 0C&
69 3C 3F 6A 30 65 66 33 33 66 65 30 6A 3F 3C 69&
03 56 55 00 5A 0F 0C 59 59 0C 0F 5A 00 55 56 03&
66 33 30 65 3F 6A 69 3C 3C 69 6A 3F 65 30 33 66&
65 30 33 66 3C 69 6A 3F 3F 6A 69 3C 66 33 30 65&
00 55 56 03 59 0C 0F 5A 5A 0F 0C 59 03 56 55 00"%
  for llIndx = 0 to SM_ECC_TABLE_SIZE-1
    bSMECCTable(llIndx) = nfroms(sSMECCTableInit$,llIndx,1)
  next
end


'' calculate ECC for 256 byte using CP0-CP5 code table[0-255]
'' bpvEcc1: LP15,LP14,LP13,...
'' bpvEcc2: LP07,LP06,LP05,...
'' bpvEcc3: CP5,CP4,CP3,...,"1","1"
sub vSMCalculateECC(long lpDataStart; var byte bpvEcc1, bpvEcc2, bpvEcc3)
  long llIndx
  byte blCode, blRegBit7, blEccBit7
  byte blReg1 '' D-all,CP5,CP4,CP3,...
  byte blReg2 '' LP14,LP12,L10,...
  byte blReg3 '' LP15,LP13,L11,...

  blReg1 = 0
  blReg2 = 0
  blReg3 = 0
  for llIndx = 0 to SM_ECC_PAGE_SIZE - 1
    blCode = nfroms( sSmPgSpareBuf$, llIndx + lpDataStart, 1 )
    blCode = bSMECCTable( blCode )
    blReg1 = blReg1 bitxor ( blCode bitand MASK_CPS )
    if ( blCode bitand BIT6 ) <> 0 then
      blReg3 = blReg3 bitxor llIndx
      blReg2 = blReg2 bitxor bitnot( llIndx )
    endif
  next

  blRegBit7 = BIT7	'' 80h=10000000b
  blEccBit7 = BIT7	'' 80h=10000000b
  bpvEcc1 = 0
  bpvEcc2 = 0
  loop 4
    if ( blReg3 bitand blRegBit7 ) <> 0 then
      bpvEcc1 = bpvEcc1 bitor blEccBit7	'' LP15,13,11,9 -> bpvEcc1
    endif
    blEccBit7 = blEccBit7 shr 1
    if ( blReg2 bitand blRegBit7 ) <> 0 then
      bpvEcc1 = bpvEcc1 bitor blEccBit7	'' LP14,12,10,8 -> bpvEcc1
    endif
    blEccBit7 = blEccBit7 shr 1
    blRegBit7 = blRegBit7 shr 1
  endloop
  blEccBit7 = BIT7	'' 80h=10000000b
  loop 4
    if ( blReg3 bitand blRegBit7 ) <> 0 then
      bpvEcc2 = bpvEcc2 bitor blEccBit7	'' LP7,5,3,1 -> bpvEcc2
    endif
    blEccBit7 = blEccBit7 shr 1
    if ( blReg2 bitand blRegBit7 ) <> 0 then
      bpvEcc2 = bpvEcc2 bitor blEccBit7	'' LP6,4,2,0 -> bpvEcc2
    endif
    blEccBit7 = blEccBit7 shr 1
    blRegBit7 = blRegBit7 shr 1
  endloop
  bpvEcc1 = bitnot( bpvEcc1 )	'' inverse bpvEcc1
  bpvEcc2 = bitnot( bpvEcc2 )	'' inverse bpvEcc2
  bpvEcc3 = ( bitnot( blReg1 ) shl 2 ) bitor BIT1BIT0	'' make TEL format
end




'' bpEcc1: LP15,LP14,LP13,...
'' bpEcc2: LP07,LP06,LP05,...
'' bpEcc3: CP5,CP4,CP3,...,"1","1"
sub bSMCorrectDataByECC( long lpDataStart, lpEccOffset; byte bpEcc1, bpEcc2, bpEcc3; var byte bpvRet )
  long llL 	'' Working llL to check llD
  long llD 	'' Result of comparison
  long llIndx	'' For counting
  long llD1, llD2, llD3		'' Result of comparison
  byte blA	'' Working for blAdd
  byte blAdd	'' Byte address of cor. DATA
  byte blB	'' Working for blBit
  byte blBit	'' Bit address of cor. DATA
  byte blCorrected	'' Corrected byte
  array alEccData(3) of byte

  for llIndx = 0 to 2
    alEccData(llIndx) = nfroms( sSmPgSpareBuf$, llIndx + lpEccOffset, 1 )
  next

  llD1 = bpEcc1 bitxor alEccData(1)
  llD2 = bpEcc2 bitxor alEccData(0)	'' Compare LP's
  llD3 = bpEcc3 bitxor alEccData(2) 	'' Comapre CP's
  llD = (llD1 shl 16) + (llD2 shl 8) + llD3
  if llD = 0 then
    bpvRet = 0
    return	'' If No error, return
  endif
  if ( ( llD bitxor ( llD shr 1 ) ) bitand CORRECTABLE ) = CORRECTABLE then
    '' If correctable
    llL = BIT23
    blAdd = 0	'' Clear parameter
    blA = BIT7
    for llIndx = 0 to 7
      '' Checking 8 blBit
      if ( llD bitand llL ) <> 0 then
        blAdd = blAdd bitor blA	'' Make byte address from LP's
      endif
      llL = llL shr 2
      blA = blA shr 1
    next
    blBit = 0 	'' Clear parameter
    blB = BIT2
    for llIndx = 0 to 2
      '' Checking 3 blBit
      if ( llD bitand llL ) <> 0 then
        blBit = blBit bitor blB 	'' Make blBit address from CP's
      endif
      llL = llL shr 2
      blB = blB shr 1
    next
    blB = BIT0
    '' Put corrected data
    blCorrected = nfroms( sSmPgSpareBuf$, lpDataStart + blAdd, 1 )
    blCorrected = blCorrected bitxor ( blB shl blBit )
    sSmPgSpareBuf$ = ntos$( sSmPgSpareBuf$, lpDataStart + blAdd, 1, blCorrected )
    bpvRet = 1
    return
  endif

  llIndx = 0	'' Clear count
  llD = llD bitand 00ffffffh	'' Masking
  while llD <> 0
    '' If llD=0 finish counting
    if ( llD bitand BIT0 ) <> 0 then
      llIndx = llIndx + 1	'' Count number of 1 blBit
    endif
    llD = llD shr 1
  endwhile
  if llIndx = 1 then
    '' If ECC error
    '' Put right ECC code
    alEccData(1) = bpEcc1
    alEccData(0) = bpEcc2
    alEccData(2) = bpEcc3
    bpvRet = 2
    return
  endif
  bpvRet = 3	'' Uncorrectable error
end


sub vSMUpdateECC()
  byte blEcc1, blEcc2, blEcc3
  string slEcc1$, slEcc2$

#ifdef FS_SECURE_INIT
  slEcc1$ = FS_STR_VARS_INIT_VALUE	'$JK
  slEcc2$ = FS_STR_VARS_INIT_VALUE	'$JK
#endif	'' FS_SECURE_INIT

  slEcc1$ = calc_ecc$( sSmPgSpareBuf$, 0, 0 )
  slEcc2$ = calc_ecc$( sSmPgSpareBuf$, 256, 0 )
  sSmPgSpareBuf$ = stos$( sSmPgSpareBuf$, lSmBytesInPage + SM_SPARE_ECC10_OFFS, slEcc1$, 3 )
  sSmPgSpareBuf$ = stos$( sSmPgSpareBuf$, lSmBytesInPage + SM_SPARE_ECC20_OFFS, slEcc2$, 3 )
#cm
  call vSMCalculateECC( 0, blEcc1, blEcc2, blEcc3 )
  sSmPgSpareBuf$ = ntos$( sSmPgSpareBuf$, lSmBytesInPage + SM_SPARE_ECC10_OFFS, 1, blEcc2 )
  sSmPgSpareBuf$ = ntos$( sSmPgSpareBuf$, lSmBytesInPage + SM_SPARE_ECC11_OFFS, 1, blEcc1 )
  sSmPgSpareBuf$ = ntos$( sSmPgSpareBuf$, lSmBytesInPage + SM_SPARE_ECC12_OFFS, 1, blEcc3 )
  call vSMCalculateECC( 256, blEcc1, blEcc2, blEcc3 )
  sSmPgSpareBuf$ = ntos$( sSmPgSpareBuf$, lSmBytesInPage + SM_SPARE_ECC20_OFFS, 1, blEcc2 )
  sSmPgSpareBuf$ = ntos$( sSmPgSpareBuf$, lSmBytesInPage + SM_SPARE_ECC21_OFFS, 1, blEcc1 )
  sSmPgSpareBuf$ = ntos$( sSmPgSpareBuf$, lSmBytesInPage + SM_SPARE_ECC22_OFFS, 1, blEcc3 )
#endcm
end

#endif  '' __FS_ECC_I_INC

