'----------------------------------------------------------------------------------------
'    Name:  Fs_Fat_SdCard_I.Inc
'    Type:  Tiger-BASIC(tm) Source Code (Basic Tiger File System)
' Release:  1.3
' Purpose:  BTFS - Implementation of FAT12/FAT16 routines that are specific for SD Card
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (English)
'	email: support@wilke.de              (German)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (English)
'	www.wilke.de                         (German)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------


#ifndef __FS_SD_CARD_I
#define __FS_SD_CARD_I

'' ********************************************
'' *       Master Boot Record structure       *
'' ********************************************

'' Boot sector
string sBootCode$(1BEh)
''four partition table structures
string sPartTable0$(16)
string sPartTable1$(16)
string sPartTable2$(16)
string sPartTable3$(16)
'' 55h signature byte + AAh signature byte
word wSignature


'' ********************************************
'' *    Partition table entry structure       *
'' ********************************************

byte bPtActivePart	'' 80h if partition active
byte bPtStartHead	'' starting head
word wPtStartCylSect	'' starting cylinder and sector
byte bPtType		'' type ID byte
byte bPtEndHead		'' ending head
word wPtEndCylSect	'' ending cylinder and sector
long lPtStart		'' starting sector
long lPtSize		'' size of partition

'' Clobale Variables
long lSDCardBytesInBlock
long lSDCardBytesInPage

'' this variable is assigned only after reading of file system configuration
'' so it's a kind of extern variable for this project file
long lDataAreaStart

string sSDCardFindBuffer$(SM_FIND_BUFFER_SIZE)

'' signature: sub bHdInit( var byte bpvRet )
sub bSDCardInit( var byte bpvRet )
  byte blResult
  long blIndex
  blResult = 0
  '' Initialize and turn on SD Card
  call vSDInit()
  call bSDIsCardDetected( blResult )
  IF blResult = 0 THEN
    blIndex = 0    	
    call vSDSetRwNCRVar( NCR )
    WHILE blIndex < 100
      call bSDResetCard( blResult )
      IF blResult = 0 THEN
	call vSDSetBlockLenVar( READ_BL_LEN_MAX )
	call vSDSetBytesInBlockVar()
	call vSDSetBytesInPageVar()
	bpvRet = TRUE
	RETURN
      ENDIF
      blIndex = blIndex + 1
    ENDWHILE
    bpvRet = FALSE
  ENDIF
end


sub lSDCardGetSize( var long lpvSDCardSize )
  byte blStatus
  lpvSDCardSize = 0

''  call bSDReadStatus( blStatus )
''  if blStatus = 0 THEN
    call bSDReadConfiguration( blStatus )
    if blStatus = 0 THEN
      call lSDCalcCardSize( lpvSDCardSize )
''''      call bSDInitCard( blStatus )
      call bSDResetCard( blStatus )
    endif
''  endif
end


sub lSDCardGetRoundedSize( var long lpvSDCardRoundedSize )
  long llSDCardSize, llSize, llNextSize
  call lSDCardGetSize( llSDCardSize )
  lpvSDCardRoundedSize = 0
  llSize = 1
  while lpvSDCardRoundedSize = 0 and llSize <= 2048M
    llNextSize = llSize shl 1
    if llSDCardSize > llSize and llSDCardSize <= llNextSize then
      lpvSDCardRoundedSize = llNextSize
    else
      llSize = llNextSize
    endif
  endwhile
end


'' For compatibility with HAL (hardware abstraction level) calls
'' signature: sub sHdReadString( long lpAddr; long lpSize; var string spvBuffer$; var byte bpvIsOk )
sub sSDCardReadString( long lpAddr; long lpSize; var string spvBuffer$; var byte bpvIsOk )
  byte blResult
#ifdef READ_MULTIPLE_BLOCK
  call bSDReadMultipleBlock(lpAddr, lpSize,  spvBuffer$, blResult)
#else
  call bSDReadBlock(lpAddr, lpSize,  spvBuffer$, blResult)
#endif
  '' Attention please	!!!
  bpvIsOk = TRUE
  if blResult <> 0 then
    bpvIsOk = FALSE
  endif
end


'' *** signature: sub lHdReadNum( long lpAddr; byte bpSize; var long lpvNum; var byte bpvIsOk )
'' bpSize cann only from 1 to 4
sub lSDCardReadNum( long lpAddr; byte bpSize; var long lpvNum; var byte bpvIsOk )
  byte blResult
  long llSize
  string slBuffer$(4)
  slBuffer$ = ""
  llSize = bpSize
  call bSDReadBlock( lpAddr, llSize, slBuffer$, blResult )
  lpvNum = nfroms( slBuffer$, 0, llSize )
  '' Attention please	!!!
  bpvIsOk = TRUE
  if blResult <> 0 then
    bpvIsOk = FALSE
  endif
end


'' For compatibility with HAL (hardware abstraction level) calls
'' signature: sub vHdWriteString( long lpAddr, lpPos, lpSize; var string spvBuffer$ )
sub vSDCardWriteString( long lpAddr, lpPos, lpSize; var string spvBuffer$ )
  BYTE blStatus
#ifdef WRITE_MULTIPLE_BLOCK
  call bSDWriteMultipleBlock(lpAddr, lpSize, spvBuffer$, blStatus)
#else
  call bSDWriteBlock(lpAddr, lpSize, spvBuffer$, blStatus)
#endif
end


sub vSDSetBytesInBlockVar()
  call lSDGetBlockLenVar( lSDCardBytesInBlock )
end


sub vSDSetBytesInPageVar()
  lSDCardBytesInPage = 512
end


sub bSDCardGetInfoSet( var string spvInfoSet$; var byte bpvIsRead )
  bpvIsRead = TRUE
  spvInfoSet$ = fill$( "<00h>",max_len(spvInfoSet$))
end


''The MBR program code starts at offset 0000.
''The MBR messages start at offset 008b.
''The partition table starts at offset 01be.
''The signature is at offset 01fe.
sub bGetMBR(var byte bpvIsFound)
  string slBuffer$(SECTOR_SIZE)

  sBootCode$ = ""
  sPartTable0$ = ""
  sPartTable1$ = ""
  sPartTable2$ = ""
  sPartTable3$ = ""

  call sHdReadString( 00h, SECTOR_SIZE, slBuffer$, bpvIsFound )

  if bpvIsFound = FALSE then
    return
  endif

  if nfroms(slBuffer$, SECTOR_SIZE-2, 2)<>0AA55h then
    bpvIsFound = FALSE
    return
  else
    wSignature = 55AAh
    sBootCode$ = stos$(sBootCode$, 0, slBuffer$, 1BEh)
    sPartTable0$ = stos$(sPartTable0$, 0, slBuffer$, 1BEh, 16)
    sPartTable1$ = stos$(sPartTable1$, 0, slBuffer$, 1CEh, 16)
    sPartTable2$ = stos$(sPartTable2$, 0, slBuffer$, 1DEh, 16)
    sPartTable3$ = stos$(sPartTable3$, 0, slBuffer$, 1EEh, 16)
  endif
end

sub vGetPartitionTable(string slPartitionTable$)
  bPtActivePart = nfroms(slPartitionTable$, 0, 1)
  bPtStartHead = nfroms(slPartitionTable$, 1, 1)
  wPtStartCylSect = nfroms(slPartitionTable$, 2, 2)
  bPtType = nfroms(slPartitionTable$, 4, 1)
  bPtEndHead = nfroms(slPartitionTable$, 5, 1)
  wPtEndCylSect = nfroms(slPartitionTable$, 6, 2)
  lPtStart = nfroms(slPartitionTable$, 8, 4)
  lPtSize = nfroms(slPartitionTable$, 12, 4)
end


'' The "chk_fnam" function can now search only in the strings that are shorter than 256 bytes;
'' the SSFDC forum papers say that the minimal size of a SM page can not be less than 256 bytes,
'' this information is used here
sub lSDCardFindStringInMemBlock( string spSearchedString$; long lpStartAddr, lpMemBlockSize, lpStepSize; var long lpvStringAddr )
  byte blIsAvailable, blIsFound
  long llCurAddr, llCurSize, llCurPagePtr, llSearchedStrLen, llPosInStr

  ''LONG lSDCardBytesInBlock
  ''LONG lSDCardBytesInPage

  blIsFound = FALSE
  lpvStringAddr = -2
  llSearchedStrLen = len(spSearchedString$)
  if llSearchedStrLen > 255 or llSearchedStrLen > lpMemBlockSize then
    return
  endif
  llCurAddr = lpStartAddr
  llCurSize = lSDCardBytesInPage - mod( llCurAddr, lSDCardBytesInPage )
  if llCurSize > lpMemBlockSize then
    llCurSize = lpMemBlockSize
  endif
  while blIsFound = FALSE and llCurAddr - lpStartAddr < lpMemBlockSize
    call sSDCardReadString( llCurAddr, llCurSize, sSDCardFindBuffer$, blIsAvailable )
    if blIsAvailable = TRUE then
      llCurPagePtr = 0
      while blIsFound = FALSE and llCurPagePtr < llCurSize
	''      llPosInStr = instr( spSearchedString$, sSmFindBuffer$, llCurPagePtr, 4 )
	llPosInStr = chk_fnam( sSDCardFindBuffer$, llCurPagePtr, lpStepSize, spSearchedString$, 0 )
	if llPosInStr >= 0 then
	  lpvStringAddr = llCurAddr + llPosInStr
	  blIsFound = TRUE
	endif
	llCurPagePtr = llCurPagePtr + 255 - llSearchedStrLen
      endwhile
    endif
    llCurAddr = llCurAddr + llCurSize
    if llCurAddr - lpStartAddr + lSDCardBytesInPage < lpMemBlockSize then
      llCurSize = lSDCardBytesInPage
    else
      llCurSize = lpMemBlockSize - ( llCurAddr - lpStartAddr )
    endif
  endwhile
end

#endif  '' __FS_SD_CARD_I

