'----------------------------------------------------------------------------------------
'    Name:  Fs_Fmt_I.Inc
'    Type:  Tiger-BASIC(tm) Source Code (Basic Tiger File System)
' Release:  1.3
' Purpose:  BTFS - Implementation of formatting process
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (English)
'	email: support@wilke.de              (German)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (English)
'	www.wilke.de                         (German)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------


#ifndef __FS_FMT_I_INC
#define __FS_FMT_I_INC

'' extra variables for settings of Master Boot Sector and Partition Boot Sector
word wBsaHostCylinders
byte bBsaHostHeads
byte bBsaHostSectors
long lBsaHostAllSectors
word wBsaClusterSize
word wBsaDirEntries
word wBsaHiddenSectors
long lBsaPartitionAll


sub bFmtSetMediaData( var word wpvCylinders; var byte bpvHeads, bpvSectors; &
  var word wpvClusterSize, wpvHiddenSectors; var byte bpvSuccess )
  long llMediaSize
  call lHdGetMediaSize( llMediaSize )
  bpvSuccess = TRUE

  switch llMediaSize
    case 1M:
      wpvCylinders = 125
      bpvHeads = 4
      bpvSectors = 4
      wpvClusterSize = 4
      wpvHiddenSectors = 13
    case 2M:
      wpvCylinders = 125
      bpvHeads = 4
      bpvSectors = 8
      wpvClusterSize = 4
      wpvHiddenSectors = 11
    case 4M:
      wpvCylinders = 250
      bpvHeads = 4
      bpvSectors = 8
      wpvClusterSize = 8
      wpvHiddenSectors = 27
    case 8M:
      wpvCylinders = 250
      bpvHeads = 4
      bpvSectors = 16
      wpvClusterSize = 8
      wpvHiddenSectors = 25
    case 16M:
      wpvCylinders = 500
      bpvHeads = 4
      bpvSectors = 16
      wpvClusterSize = 16
      wpvHiddenSectors = 41
    case 32M:
      wpvCylinders = 500
      bpvHeads = 8
      bpvSectors = 16
      if bFATFmtWinFlag = TRUE then
        wpvClusterSize = 1
      else
        wpvClusterSize = 16
      endif
      wpvHiddenSectors = 35
    case 64M:
      wpvCylinders = 500
      bpvHeads = 8
      bpvSectors = 32
      wpvClusterSize = 16
      wpvHiddenSectors = 55
    case 128M:
      wpvCylinders = 500
      bpvHeads = 16
      bpvSectors = 32
      wpvClusterSize = 16
      wpvHiddenSectors = 47
    default:
      wpvCylinders = 0
      bpvHeads = 0
      bpvSectors = 0
      wpvClusterSize = 0
      wpvHiddenSectors = 0
      bpvSuccess = FALSE
  endswitch
end


sub bFmtSetBootParameters( var byte bpvSuccess )
  bpvSuccess = TRUE
#ifdef SD_CARD
  call bSDFmtSetMediaData( wBsaHostCylinders, bBsaHostHeads, bBsaHostSectors, &
    wBsaClusterSize, wBsaHiddenSectors, lBsaHostAllSectors, wBsaDirEntries, bpvSuccess )
#endif
#ifdef SMC_CARD
  call bFmtSetMediaData( wBsaHostCylinders, bBsaHostHeads, bBsaHostSectors, &
    wBsaClusterSize, wBsaHiddenSectors, bpvSuccess )
  if bpvSuccess = FALSE then
    return
  endif
  lBsaHostAllSectors = wBsaHostCylinders
  lBsaHostAllSectors = lBsaHostAllSectors * bBsaHostHeads * bBsaHostSectors
  if bFATFmtWinFlag = FALSE then
    wBsaClusterSize = wBsaClusterSize * ( 1K / HD_SECT_SIZE )
    wBsaDirEntries = DIR_ENTRIES_NUM
  else
    wBsaDirEntries = DIR_ENTRIES_NUM_WIN
  endif
#endif
  lBsaPartitionAll = lBsaHostAllSectors - wBsaHiddenSectors
end


sub vFmtSetMBSectorData()
  bMbx86boot = MB_PARTITION_BOOTABLE
  bMbStartHead = wBsaHiddenSectors / bBsaHostSectors
  bMbStartSector = mod( wBsaHiddenSectors, bBsaHostSectors ) + 1
  bMbStartCylinder = 00h
  if lBsaPartitionAll / wBsaClusterSize > MAX_CLUSTERS_FAT12 then
    bMbPartitionType = MB_PARTITION_TYPE_FAT16
  else
    bMbPartitionType = MB_PARTITION_TYPE_FAT12
  endif
  bMbEndHead = bBsaHostHeads - 1
  bMbEndSector = ( ( ( wBsaHostCylinders - 1 ) / 4 ) bitand 0C0h ) + bBsaHostSectors
  bMbEndCylinder = ( wBsaHostCylinders - 1 ) bitand 0FFh
  lMbLogStartSector = wBsaHiddenSectors
  lMbNumSectors = lBsaHostAllSectors - wBsaHiddenSectors
  wMbSignature = BOOT_MAGIC
end


'' fill a string with the master boot sector data
sub sFmtMakeMBSector( var string spvMBSector$ )
  long llPart1Offs
  llPart1Offs = MB_PART_1_INFO_OFFS
  spvMBSector$ = fill$( "00"%, HD_SECT_SIZE )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_X86BOOT_POS, MBR_X86BOOT_SIZE, bMbx86boot )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_START_HEAD_POS, MBR_START_HEAD_SIZE, bMbStartHead )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_START_SECT_POS, MBR_START_SECT_SIZE, bMbStartSector )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_START_CYLINDER_POS, MBR_START_CYLINDER_SIZE, bMbStartCylinder )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_PARTITION_TYPE_POS, MBR_PARTITION_TYPE_SIZE, bMbPartitionType )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_END_HEAD_POS, MBR_END_HEAD_SIZE, bMbEndHead )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_END_SECT_POS, MBR_END_SECT_SIZE, bMbEndSector )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_END_CYLINDER_POS, MBR_END_CYLINDER_SIZE, bMbEndCylinder )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_LOG_START_SECT_POS, MBR_LOG_START_SECT_SIZE, lMbLogStartSector )
  spvMBSector$ = ntos$( spvMBSector$, llPart1Offs + MBR_NUM_SECT_POS, MBR_NUM_SECT_SIZE, lMbNumSectors )
  spvMBSector$ = ntos$( spvMBSector$, BOOT_MAGIC_POS, BOOT_MAGIC_SIZE, wMbSignature )
end


sub vFmtSetPBSectData()
''  sBsJmp$ = "e9 00 00"%

#ifdef FS_SECURE_INIT
  sBsOemName$ = FS_STR_VARS_INIT_VALUE	'$JK
#endif	'' FS_SECURE_INIT

  sBsJmp$ = BP_JUMP_INSTR
  sBsOemName$ = fill$( BP_OEM_NAME, 8 )
  wBpbBytesPerSector = HD_SECT_SIZE
  bBpbSectorsPerCluster = wBsaClusterSize
  wBpbReservedSectors = 1
  bBpbNumberOfFATs = BP_NUM_OF_FAT
  wBpbRootEntries = wBsaDirEntries
  if lBsaHostAllSectors > 0FFFFh then
    lBpbTotalSectors = 0000h
  else
    lBpbTotalSectors = lBsaHostAllSectors - wBsaHiddenSectors
  endif
  bBpbMedia = BP_MEDIA_ID_ANY
  if lBsaPartitionAll / wBsaClusterSize > MAX_CLUSTERS_FAT12 or bFATFmtWinFlag = TRUE then
    wBpbSectorsPerFAT = ( ( lBsaPartitionAll / wBsaClusterSize ) * 4 ) / 1K + 1
  else
    wBpbSectorsPerFAT = ( ( lBsaPartitionAll / wBsaClusterSize ) * 3 ) / 1K + 1
  endif

  wBpbSectorsPerTrack = bBsaHostSectors
  wBpbHeadsPerCylinder = bBsaHostHeads
  lBpbHiddenSectors = wBsaHiddenSectors
  if lBsaHostAllSectors > 0FFFFh then
    lBpbTotalSectorsBig = lBsaHostAllSectors - wBsaHiddenSectors
  else
    lBpbTotalSectorsBig = 0
  endif

  bBsDriveNumber = 0h
  bBsUnused = 0h
  bBsExtBootSignature = 0h
  lBsSerialNumber = 0h
  sBsVolumeLabel$ = fill$( "00"%, 11 )
  if lBsaPartitionAll / wBsaClusterSize > MAX_CLUSTERS_FAT12 or bFATFmtWinFlag = TRUE then
    sBsFileSystem$ = FAT16_NAME_EXT
  else
    sBsFileSystem$ = FAT12_NAME_EXT
  endif
end


'' fill a string with the partition boot sector data
sub sFmtMakePBSector( var string spvPBSector$ )
  spvPBSector$ = fill$( "00"%, HD_SECT_SIZE )
  spvPBSector$ = stos$( spvPBSector$, BS_JMP_POS, sBsJmp$, BS_JMP_SIZE )
  spvPBSector$ = stos$( spvPBSector$, BS_OEM_NAME_POS, sBsOemName$, BS_OEM_NAME_SIZE )

  spvPBSector$ = ntos$( spvPBSector$, BPB_BYTES_PER_SECT_POS, &
    BPB_BYTES_PER_SECT_SIZE, wBpbBytesPerSector )
  spvPBSector$ = ntos$( spvPBSector$, BPB_SECT_PER_CLUSTER_POS, &
    BPB_SECT_PER_CLUSTER_SIZE, bBpbSectorsPerCluster )
  spvPBSector$ = ntos$( spvPBSector$, BPB_RESERVED_SECT_POS, &
    BPB_RESERVED_SECT_SIZE, wBpbReservedSectors )
  spvPBSector$ = ntos$( spvPBSector$, BPB_NUMBER_OF_FATS_POS, &
    BPB_NUMBER_OF_FATS_SIZE, bBpbNumberOfFATs )
  spvPBSector$ = ntos$( spvPBSector$, BPB_ROOT_ENTRIES_POS, &
    BPB_ROOT_ENTRIES_SIZE, wBpbRootEntries )
  spvPBSector$ = ntos$( spvPBSector$, BPB_TOTAL_SECT_POS, &
    BPB_TOTAL_SECT_SIZE, lBpbTotalSectors )
  spvPBSector$ = ntos$( spvPBSector$, BPB_MEDIA_POS, &
    BPB_MEDIA_SIZE, bBpbMedia )
  spvPBSector$ = ntos$( spvPBSector$, BPB_SECT_PER_FAT_POS, &
    BPB_SECT_PER_FAT_SIZE, wBpbSectorsPerFAT )
  spvPBSector$ = ntos$( spvPBSector$, BPB_SECT_PER_TRACK_POS, &
    BPB_SECT_PER_TRACK_SIZE, wBpbSectorsPerTrack )
  spvPBSector$ = ntos$( spvPBSector$, BPB_HEADS_PER_CYL_POS, &
    BPB_HEADS_PER_CYL_SIZE, wBpbHeadsPerCylinder )
  spvPBSector$ = ntos$( spvPBSector$, BPB_HIDDEN_SECT_POS, &
    BPB_HIDDEN_SECT_SIZE, lBpbHiddenSectors )
  spvPBSector$ = ntos$( spvPBSector$, BPB_TOTAL_SECT_BIG_POS, &
    BPB_TOTAL_SECT_BIG_SIZE, lBpbTotalSectorsBig )

  spvPBSector$ = ntos$( spvPBSector$, BS_DRIVE_NUMBER_POS, BS_DRIVE_NUMBER_SIZE, bBsDriveNumber )
  spvPBSector$ = ntos$( spvPBSector$, BS_UNUSED_POS, BS_UNUSED_SIZE, bBsUnused )
  spvPBSector$ = ntos$( spvPBSector$, BS_EXT_BOOT_SIGN_POS, BS_EXT_BOOT_SIGN_SIZE, bBsExtBootSignature )
  spvPBSector$ = ntos$( spvPBSector$, BS_SERIAL_NUMBER_POS, BS_SERIAL_NUMBER_SIZE, lBsSerialNumber )
  spvPBSector$ = stos$( spvPBSector$, BS_VOLUME_LABEL_POS, sBsVolumeLabel$, BS_VOLUME_LABEL_SIZE )
  spvPBSector$ = stos$( spvPBSector$, BS_FILE_SYSTEM_POS, sBsFileSystem$, BS_FILE_SYSTEM_SIZE )
  spvPBSector$ = ntos$( spvPBSector$, BOOT_MAGIC_POS, BOOT_MAGIC_SIZE, wMbSignature )
end

#endif  '' __FS_FMT_I_INC

