'----------------------------------------------------------------------------------------
'    Name:  Fs_Hal_D.Inc
'    Type:  Tiger-BASIC(tm) Source Code (Basic Tiger File System)
' Release:  1.3
' Purpose:  BTFS - Definition of Hardware Abstraction Layer
' 	    HAL is used to simplify the adaptation of the file system subroutines
' 	    to the working with other storage devices
'
'  (C) - Copyright Wilke Technology, P.O.Box 1727, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'	Wilke Technology GmbH
'	The Tiger Support Team
'	P.O.Box 1727,       D-52018 Aachen, Germany
'	Krefelder Str. 147, D-52070 Aachen, Germany
'
'	email: support@wilke-technology.com  (English)
'	email: support@wilke.de              (German)
'	Phone: +49 (241) 918 900    Mo to Fr, 7:00 to 16:00 (GMT)
'	Fax:   +49 (241) 918 9068
'
' New information, new drivers and free downloads see:
'
'	www.wilke-technology.com	     (English)
'	www.wilke.de                         (German)
'
' Sincerely,
'
'	Your Tiger Support Team
'
'----------------------------------------------------------------------------------------


#ifndef __FS_HAL_D_INC
#define __FS_HAL_D_INC

''*******************************************************************************
#ifdef SMC_CARD
'' Smart Media
'' signature: sub lHdGetMediaSize( var long lpvMediaSize )
#define lHdGetMediaSize  lSMGetMediaSize

'' *** signature: sub lHdReadNum( long lpAddr; byte bpSize; var long lpvNum; var byte bpvIsOk )
#define lHdReadNum  lSMReadNum

'' signature: sub lHdReadNumPhys( long lpPhysAddr; byte bpSize; var long lpvNum )
#define lHdReadNumPhys  lSMReadNumPhys

'' *** signature: sub sHdReadString( long lpAddr; long lpSize; var string spvBuffer$; var byte bpvIsOk )
#define sHdReadString  sSMReadString

'' signature: sub sSMReadStringPhys( long lpPhysAddr, lpLogAddr; long lpSize; var string spvBuffer$ )
#define sHdReadStringPhys sSMReadStringPhys

'' *** signature: sub vHdWriteString( long lpAddr, lpPos, lpSize; var string spvBuffer$ )
#define vHdWriteString  vSMWriteString

'' *** signature: sub bHdInit( var byte bpvRet )
#define bHdInit   bSMInit

'' *** signature: sub lSMFindStringInMemBlock( string spSearchedString$; long lpStartAddr, &
''  lpMemBlockSize, lpStepSize; var long lpvStringAddr )
#define lHdFindStringInMemBlock lSMFindStringInMemBlock

'' signature: sub bSMGetInfoSet( var string spvInfoSet$; var byte bpvIsRead )
#define bHdGetInfoSet  bSMGetInfoSet

'' *** it is a global variable, not a subroutine
#define lHdBytesInBlock  lSmBytesInBlock

'' *** it is a global variable, not a subroutine
#define lHdBytesInPage  lSmBytesInPage

#endif '' END DEFINED SMC_CARD

''****************************************************
''*                    SD_CARD                       *
''****************************************************
#ifdef SD_CARD

#define bHdGetInfoSet   bSDCardGetInfoSet

#define lHdGetMediaSize lSDCardGetSize

#define lHdGetMediaRoundedSize lSDCardGetRoundedSize

#define lHdReadNum  lSDCardReadNum

#define sHdReadString  sSDCardReadString

#define vHdWriteString  vSDCardWriteString

#define bHdReadBlock  bSDReadBlock

#define bHdWriteBlock  bSDWriteBlock

#define bHdInit   bSDCardInit

#define lHdFindStringInMemBlock lSDCardFindStringInMemBlock

'' *** it is a global variable, not a subroutine
#define lHdBytesInBlock  lSDCardBytesInBlock

'' *** it is a global variable, not a subroutine
#define lHdBytesInPage  lSDCardBytesInPage

#endif

''*******************************************************************************

#define HD_SECT_SIZE		SM_MAX_PAGE_SIZE
'' bytes used to fill erased sectors
#define HD_ERASED_BYTE_ROOT_STR	SM_ERASED_BYTE_ROOT_STR
#define HD_ERASED_BYTE_STR	SM_ERASED_BYTE_STR


'' Offsets and sizes of the fields of the hardware info set
#define HDI_MAKER_CODE_POS	SM_MAKER_CODE_POS	' <1> Maker-Code = Manufacturer
#define HDI_ID_CODE_POS		SM_ID_CODE_POS		' <1> ID-Code = Card-Id (Memory Size)
#define HDI_BYTES_IN_SPARE_POS	SM_BYTES_IN_SPARE_POS	' <2> Nr. of Bytes in Spare-Field (normal: 16 Bytes)
#define HDI_BYTES_IN_PAGE_POS	SM_BYTES_IN_PAGE_POS	' <4> Nr. of DATA Bytes in a Page (without spare-field)
#define HDI_PAGES_IN_BLOCK_POS	SM_PAGES_IN_BLOCK_POS	' <2> Nr. of Pages in a Block
#define HDI_BYTES_IN_BLOCK_POS	SM_BYTES_IN_BLOCK_POS	' <4> Nr. of DATA-Bytes in a Block
#define HDI_NO_OF_BLOCKS_POS	SM_NO_OF_BLOCKS_POS	' <2> Total Number of "Blocks" in this SmartMedia Card
#define HDI_ADR_HIGH_BLOCK_POS	SM_ADR_HIGH_BLOCK_POS	' <4> Base ADR of highest Block
#define HDI_ADR_END_POS		SM_ADR_END_POS		' <4> End-ADR = 1.ADR after the last Byte !!

#define HDI_MAKER_CODE_SIZE	SM_MAKER_CODE_SIZE	' <1> Maker-Code = Manufacturer
#define HDI_ID_CODE_SIZE   	SM_ID_CODE_SIZE		' <1> ID-Code = Card-Id (Memory Size)
#define HDI_BYTES_IN_SPARE_SIZE	SM_BYTES_IN_SPARE_SIZE	' <2> Nr. of Bytes in Spare-Field (normal: 16 Bytes)
#define HDI_BYTES_IN_PAGE_SIZE	SM_BYTES_IN_PAGE_SIZE	' <4> Nr. of DATA Bytes in a Page (without spare-field)
#define HDI_PAGES_IN_BLOCK_SIZE	SM_PAGES_IN_BLOCK_SIZE	' <2> Nr. of Pages in a Block
#define HDI_BYTES_IN_BLOCK_SIZE	SM_BYTES_IN_BLOCK_SIZE	' <4> Nr. of DATA-Bytes in a Block
#define HDI_NO_OF_BLOCKS_SIZE	SM_NO_OF_BLOCKS_SIZE	' <2> Total Number of "Blocks" in this SmartMedia Card
#define HDI_ADR_HIGH_BLOCK_SIZE	SM_ADR_HIGH_BLOCK_SIZE	' <4> Base ADR of highest Block
#define HDI_ADR_END_SIZE   	SM_ADR_END_SIZE		' <4> End-ADR = 1.ADR after the last Byte !!

'' Size of the hardware info set
#define HDI_BLOCK_SIZE		SMCARD_INFO_BLOCK_SIZE

#endif  '' __FS_HAL_D_INC

